//
//  Entities.swift
//  GraphPad
//
//  Created by Joe Gregg on 10/22/24.
//

import Foundation
import SwiftUI
import SwiftData

@Model 
class Vertex {
    @Attribute(.unique) var id: String = UUID().uuidString
    var centerX : Float
    var centerY : Float
    let radius : Float = 5.0
    var offsetX : Float = 0.0
    var offsetY : Float = 0.0
    
    init(centerX : Float,centerY: Float) {
        self.centerX = centerX
        self.centerY = centerY
    }
    
    func getCenter() -> CGPoint {
        return CGPoint(x:CGFloat(centerX + offsetX),y:CGFloat(centerY + offsetY))
    }
    
    func drawIn(_ context: GraphicsContext) {
        let circle = Path(ellipseIn: CGRect(x: CGFloat(centerX + offsetX - radius), y: CGFloat(centerY + offsetY - radius), width: CGFloat(2 * radius), height: CGFloat(2 * radius)))
        context.fill(circle,with: .color(.red))
    }
    
    func containsPoint(_ pt : CGPoint) -> Bool {
        let deltaX = (centerX - Float(pt.x))
        let deltaY = (centerY - Float(pt.y))
        let distSquared = deltaX*deltaX + deltaY*deltaY
        if distSquared < radius*radius {
            return true
        } else {
            return false
        }
    }
    
    func setOffset(_ offset : CGSize) {
        self.offsetX = Float(offset.width)
        self.offsetY = Float(offset.height)
    }
    
    func endDrag() {
        centerX += offsetX
        centerY += offsetY
        offsetX = 0.0
        offsetY = 0.0
    }
}

@Model
class Edge {
    @Attribute(.unique) var id: String = UUID().uuidString
    var origin : Vertex
    var dest : Vertex
    
    init(origin: Vertex, dest: Vertex) {
        self.origin = origin
        self.dest = dest
    }
    
    func drawIn(_ context: GraphicsContext) {
        let edge = Path() { path in
            path.move(to: origin.getCenter())
            path.addLine(to: dest.getCenter())
        }
        context.stroke(edge,with:.color(.green),lineWidth:2)
    }
}

@Model
class Graph {
    @Attribute(.unique) var id: String = UUID().uuidString
    var name : String
    var created : Date
    var vertices : [Vertex]
    var edges : [Edge]
    var dragging : Vertex?
    var started : Bool = false
    
    init(name: String,created: Date) {
        self.name = name
        self.created = created
        vertices = []
        edges = []
    }
    
    func makeVertex(_ center : CGPoint) {
        let v = Vertex(centerX: Float(center.x), centerY: Float(center.y))
        vertices.append(v)
    }
    
    func startDrag(_ pt : CGPoint) {
        started = true
        for v in vertices {
            if v.containsPoint(pt) {
                dragging = v
            }
        }
    }
    
    func continueDrag(_ offset : CGSize) {
        if let s = dragging {
            s.setOffset(offset)
        }
    }
    
    func endDrag() {
        if let s = dragging {
            s.endDrag()
        }
        dragging = nil
        started = false
    }
    
    func endDraw(_ pt : CGPoint) {
        for v in vertices {
            if v.containsPoint(pt) && (v !== dragging) {
                if let start = dragging {
                    let newEdge = Edge(origin:start,dest:v)
                    edges.append(newEdge)
                }
                break
            }
        }
        started = false
        dragging = nil
    }
}
